#On Error Resume Next
#***********************************************************BeginFile************
# Copyright:  (c) 2001 by Consul Risk Management B.V.  All Rights Reserved
# Time-stamp: <010924 Edwin 10:37 getnewrecs.vbs>
# Name:
#   getnewrecs.vbs
# Purpose:
#   retrieve records appended to a log since the previous run
#   and store them sequentially in a new file
# Notes:
#   getnewrecs.vbs <outfilename> <platformname> <directory> <filename pattern>
#   <outfilename>       Name of temp chunk To be collected
#  <platformname>      unique registry key value To store state of already collected Data
#   <directory>         path of directory To collect
#   <filename pattern>  regex of files To be collected from the directory
# History:
#   010515 0.0.1 edwin EE105004: new
#   010528 0.0.10 edwin EE105004: added functionality to read current (open) logfiles as well as old
#                                 already closed files according To a filename pattern.
#   010809 0.0.16 Edwin QE108002: write first record if the logfile is switched
#***********************************************************EndFile******
debugLog="../log/getnewrecs.log"
tracing="1"
pnum=$#
inifile="../actuatorlog.cfg"

#Initiate logging file

if [ $tracing = "1" ]
then
  touch $debugLog
  echo "Opened log file $debugLog" >> $debugLog
  if [[ $# -ge 4 ]]
  then
    OutFileName=$1
    PltfName=$2
    Directory=$3
    FilenamePattern=$4
  fi
  acnt=1
  while [[ $# -ge 1 ]]
  do
    echo "Args $acnt "$1 >> $debugLog
    shift
    acnt=$(expr $acnt + 1)
  done
fi

if [[ $pnum -le 3 ]]
then
  if [ $tracing = "1" ]
  then
    echo "Insufficient number of arguments." >> $debugLog
  fi
  exit 1
fi

previous="1970 Jan 01 00 00"
firstrecord="first run"
lastline=1

if [ -e $inifile ]
then
  if [[ ! -z `cat $inifile | grep "^$PltfName:"` ]]
  then
#get timestamp from previous run1
    previous=`cat $inifile | grep "^$PltfName:" | sed -e "s/^$PltfName:\([^:][^:]*\):.*$/\1/"`
    echo "Last date from previous run: $previous" >> $debugLog
# get last linenumber from previous run
    lastline=`cat $inifile | grep "^$PltfName:" | sed -e "s/^$PltfName:[^:][^:]*:\([0-9][0-9]*\):.*$/\1/"`
#get first record from previous run
    firstrecord=`cat $inifile | grep "^$PltfName:" | sed -e "s/^$PltfName:[^:][^:]*:[0-9][0-9]*:\(.*\)$/\1/"`
  fi
else
  touch $inifile
fi
#open outfile
printf "" > $OutFileName

#compare dates
compv() {
  if [[ $1 > $2 ]] 
  then 
    echo "0"
  else
    echo "1"
  fi
}

mounth2num () {  
  case $1 in
    'Jan')
       outval="01"
       ;;
    'Fed')
       outval="02"
       ;;
    'Mar')
       outval="03"
       ;;
    'Apr')
       outval="04"
       ;;
    'May')
       outval="05"
       ;;
    'Jun')
       outval="06"
       ;;
    'Jul')
       outval="07"
       ;;
    'Aug')
       outval="08"
       ;;
    'Sep')
       outval="09"
       ;;
    'Oct')
       outval="10"
       ;;
    'Nov')
       outval="11"
       ;;
    'Dec')
       outval="12"
       ;;
     *)
       outval="00"
       ;;
  esac
  echo $outval
}

absdate () {
  iM2=`echo $1 | sed -e 's/^\([A-Z][a-z][a-z]\).*/\1/'`
  id2=`echo $1 | sed -e 's/^[A-Z][a-z][a-z] \([0-3][0-9]\) .*/\1/'`
  id2=`echo $id2 | sed -e 's/^[A-Z][a-z][a-z] \([0-9]\) .*/0\1/'`
  iM2=`mounth2num $iM2`
  if [[ `echo "$1" | grep ":"` = "$1" ]]
  then
    ih2=`echo $1 | sed -e 's/^.*\([0-2][0-9]\):[0-5][0-9]$/\1/'`
    im2=`echo $1 | sed -e 's/^.*:\([0-5][0-9]\)$/\1/'`
    iy2=`date +%Y`
    if [[ 'date +%m' < "07" ]]
    then
      if [[ "$iM2" < "06" ]]
      then
        iy2=$(expr ${iy2} - 1)
      fi
    fi
  else
    iy2=`echo $1 | sed -e 's/^.*\([1-2][0-9][0-9][0-9]\)$/\1/'`
    ih2='23'
    im2='59'
  fi
  echo "$iy2 $iM2 $id2 $ih2 $im2"
}

cmpdate () {
  y1=`echo $2 | sed -e 's/^\([1-2][0-9][0-9][0-9]\).*/\1/'`
  M1=`echo $2 | sed -e 's/^[1-2][0-9][0-9][0-9] \([A-Z][a-z][a-z]\).*/\1/'`
  d1=`echo $2 | sed -e 's/^[1-2][0-9][0-9][0-9] [A-Z][a-z][a-z] \([0-3][0-9]\).*/\1/'`
  h1=`echo $2 | sed -e 's/^[1-2][0-9][0-9][0-9] [A-Z][a-z][a-z] [0-3][0-9] \([0-2][0-9]\).*/\1/'`
  m1=`echo $2 | sed -e 's/.*\([0-5][0-9]\)$/\1/'`
  M2=`echo $3 | sed -e 's/^\([A-Z][a-z][a-z]\).*/\1/'`
  d2=`echo $3 | sed -e 's/^[A-Z][a-z][a-z] \([0-3][0-9]\) .*/\1/'`
  d2=`echo $d2 | sed -e 's/^[A-Z][a-z][a-z] \([0-9]\) .*/0\1/'`
  M1=`mounth2num $M1`
  M2=`mounth2num $M2`
  if [ $1 = "1" ]
  then
    h2=`echo $3 | sed -e 's/^.*\([0-2][0-9]\):[0-5][0-9]$/\1/'`
    m2=`echo $3 | sed -e 's/^.*:\([0-5][0-9]\)$/\1/'`
    y2=`date +%Y`
    if [[ 'date +%m' < "07" ]]
    then
      if [[ "$M2" < "06" ]]
      then
        y2=$(expr ${y2} - 1)
      fi
    fi
  else
    y2=`echo $3 | sed -e 's/^.*\([1-2][0-9][0-9][0-9]\)$/\1/'`
    h2='23'
    m2='59'
  fi
  if [ $y1 = $y2 ]
  then
    if [ $M1 = $M2 ]
    then
      if [ $d1 = $d2 ]
      then
        if [ $h1 = $h2 ]
        then
          echo `compv $m1 $m2`
        else
          echo `compv $h1 $h2`
        fi
      else
        echo `compv $d1 $d2`
      fi
    else
      echo `compv $M1 $M2`
    fi
  else
    echo `compv $y1 $y2`
  fi
}

#get the filenames to process
#filenames can't consists spaces!!!!

buffer=""
filedate=""
flg="0"
for aa in `ls -got $Directory | grep "^-" | cut -c24-`
do
  case $flg in
   "0")
     filedate=$aa
     flg="1"
     ;;
   "1")
     case $aa in
     [1-3][0-9])
       filedate=$filedate" "$aa
       ;;
     [0-9])
       filedate=$filedate" 0"$aa
       ;;
     esac
     flg="2"
     ;;
   "2")
     case $aa in
     [0-2][0-9]:[0-5][0-9])
       filedate=$filedate" "$aa
       if [ `cmpdate 1 "$previous" "$filedate"` = "0" ]
       then
         break
       fi
       ;;
     [1-2][0-9][0-9][0-9])
       filedate=$filedate" "$aa
       if [ `cmpdate 2 "$previous" "$filedate"` = "0" ]
       then
         break
       fi
       ;;
     esac
     flg="3"
     ;;
   "3")
     if [[ ! -z `echo "$aa" | grep "$FilenamePattern"` && $aa != $OutFileName ]]
     then
       buffer="$aa $buffer"
     fi     
     flg="0"
     ;;
  esac
done

#do the actual collection of the new records
for aa in $buffer
do
#compare if equal to line in registry (same file)
#start at first record if different
  record=`head -1 $aa`
  if [ "$record" != "$firstrecord" ]
  then
    lastline=1
    firstrecord=$record
    echo "$record" >> $OutFileName
  fi
  tmpd=`ls -got $aa | cut -c24-35`
  echo "$tmpd $Directory/$aa line: $lastline" >> $debugLog
  previous=`absdate "$tmpd"`
  nline=$(expr $lastline + 1)
  tail +$nline $aa >> $OutFileName
#get last linenumber of infile
  lastline=`wc -l $aa | cut -c1-8 | sed -e 's/^ *\([^ ][^ ]*\)$/\1/'`
done

#store new values 
if ! { cat $inifile | grep -v "^${PltfName}:" > $inifile ; echo "$PltfName:$previous:$lastline:$firstrecord" >> $inifile ; }
then
  exit 1
fi