/**********************************************************BeginFile************
* Copyright:  (c) 2002 by Consul Risk Management B.V.  All Rights Reserved
* Time-stamp: <020618 Louis 14:40 cal.c>
*
* Name:  cal.c
* Purpose:
*   Functions dealing with dates and times.
* Notes:
*   
* History:
*   020121 <Version> louis <ID>: Created.
***********************************************************EndFile*************/
#include <string.h>
#include <stdlib.h>

/**********************************************************BeginFunction********
* Name: ymd2time
* Syntax:
*  int ymd2time(int year, int month, int day) 
* Parameters:
*   int year  - the year number
*   int month - the month number (1 = January)
*   int day   - the day number
* Return Value: the time_t value corresponding to the beginning of the
*   indicated day
*   
* History:
*   020121 0.0.13 louis <ID>: Created.
***********************************************************EndFunction*********/
int ymd2time(int year, int month, int day) {
  day += (1461*(year + 4800 + (month - 14)/12))/4
    + (367*(month - 2 - 12*((month - 14)/12)))/12
    - (3*((year + 4900 + (month - 14)/12)/100))/4;
  return (day - 2472663)*86400;
}

/**********************************************************BeginFunction********
* Name: date2time
* Syntax:
*  
*  int date2time(int date) 
* Parameters:
*   int date = the date, in the form of 10000*year + 100*month + day
* Return Value: the time_t value corresponding to the beginning of the
*   indicated day
* History:
*   010920 0.0.4 Louis <ID>: Created.
*   020121 0.0.13 louis <ID>: spun off the ymd() part
***********************************************************EndFunction*********/
int date2time(int date) {
  int year, month, day;

  year = date/10000;
  date = date % 10000;
  month = date/100;
  day = date % 100;
  return ymd2time(year, month, day);
}

/**********************************************************BeginFunction********
* Name: cal2time
* Syntax:
*  int cal2time(char *text) 
* Parameters:
*   char *text = the text string to be converted to a time_t value
* Return Values: the time_t value corresponding to the date in the text, or 0
*   
* History:
*   020121 0.0.13 louis <ID>: Created.
***********************************************************EndFunction*********/
int cal2time(char *text) {
  int year, month, day, hour = 0, min = 0, sec = 0;
  char *p;

  year = strtol(text, &p, 10);
  if (p == text)                /* no year found */
    return 0;
  text = p + strcspn(p, "0123456789 \t\n");
  if (!isdigit(*text))          /* no month found */
    return 0;
  month = strtol(text, &p, 10);
  text = p + strcspn(p, "0123456789 \t\n");
  if (!isdigit(*text))          /* no day found */
    return 0;
  day = strtol(text, &p, 10);
  text = p + strcspn(p, "0123456789 \t\n");
  if (!isdigit(*text))          /* no hour found */
    goto cal2time_1;
  hour = strtol(text, &p, 10);
  text = p + strcspn(p, "0123456789 \t\n");
  if (!isdigit(*text))          /* no minute found */
    goto cal2time_1;
  min = strtol(text, &p, 10);
  text = p + strcspn(p, "0123456789 \t\n");
  if (!isdigit(*text))          /* no second found */
    goto cal2time_1;
  sec = strtol(text, &p, 10);
  cal2time_1:  
  return ymd2time(year, month, day) + hour*3600 + min*60 + sec;
}

