Option Explicit      ' Force explicit variable declaration.
On Error Resume Next
'***********************************************************BeginFile************
' Copyright:  (c) 2001 by Consul Risk Management B.V.  All Rights Reserved
' Time-stamp: <020610 Ide 15:41 getsaplogs.vbs>
' Name:
'   getsaplogs.vbs
' Purpose:
'   retrieve files modified since the strPrevious run.
'   and store them sequentially in a new file
' Notes:
'   getsaplogs.vbs <outfilename> <directory> <basename> <propsfile> <version>
' History:
'    010618 0.0.1 IN Created
'    010827 0.0.23 Mostert QE108008: merge in steps when cmdline gets long
'    VL 2004jul29 updated: WE0406044 map: proper SAP 4.7 support.
'                          Added support of fifth argument, there is a SAP version
'                          Pass it to sapmerge and sapchop executables.
'***********************************************************EndFile******
Dim WshShell          ' The shell to direct commands to
Dim intArgs           ' The arguments of this program
Dim strOutFileName    ' The file containing the collected log data
Dim objDebugLogObject ' For debug purposes
Dim objDebugLogFile   ' For debug purposes
Dim intReturnCode     ' Result of a command
Dim intCount          ' Counter for arguments
Dim strLastFileMerged ' Name of last collected SAP R/3 log
Dim strLogLastFile    ' Name of last collected SAP R/3 log
Dim strLogDirectory   ' Directory of SAP R/3 logfiles
Dim strBaseName       ' Base name of SAP R/3 audit log
Dim strRegKey         ' Registry Key for last-file storage
Dim dtmPrevious       ' Timestamp of previous run ( in date format )
Dim strCmdLine        ' CommandLine
Dim objFso            ' File System Object to retreive logfiles
Dim objFolder         ' Folder Object 
Dim objFc             ' Files Collection Object
Dim objFile           ' File Object
Dim strPropsFilename  ' Name of properties file
Dim strSapVersion     ' 2004jul29VL WE0406044 added/SAP version variable

Dim strLogFile        '2006Jan06 QE0601001
Dim strStartDate,strFinishDate
Dim f,data,pos
Dim dtmEndDate

Err.Clear

Set WshShell = wscript.CreateObject("WScript.Shell")

' Constant declaration
Const conRegKey = "HKLM\Software\CONSUL Risk Management\CEAActuator\SAPR3\"
Const debugLog = "..\log\SAPR3Act.Log"
Const conTracing = True 'False
Const conConcantCmd = "..\bin\sapmerge.exe"
Const conChopperCmd = "..\bin\sapchop.exe"
Const conTmpRawData = "..\log\sapdata.raw"
Const ForReading = 1, ForWriting = 2, ForAppending = 8

' Delete old temporary file, if present
Set objFso = CreateObject("Scripting.FileSystemObject")
objFso.DeleteFile(conTmpRawData)

' Initiate logging file
Set objDebugLogObject = CreateObject("Scripting.FileSystemObject")
Set objDebugLogFile = objDebugLogObject.CreateTextFile(debugLog, True)

' Read Arguments
Set intArgs = wscript.Arguments

 For intCount = 0 To intArgs.Count - 1
     Call LogInfo( "Input Parameter " & intCount & " : " & intArgs(intCount))
 Next


If intArgs.Count <> 5 Then ' 2004jul29VL WE0406044 changed/accepting 5 arguments
    Call LogInfo("Too little arguments" )
    Call objDebugLogFile.Close
    wscript.quit 1
End If


' Read Arguments
strOutFileName = intArgs(0)
strLogDirectory = intArgs(1) 
strBaseName = intArgs(2)
strPropsFilename = intArgs(3)
strSapVersion = intArgs(4) ' 2004jul29VL WE0406044 added/SAP version variable initialization

strLogFile = "..\log\" & strPropsFilename & ".log"
Call LogInfo( "See details in log file: " & strLogFile )
Call LogInfo( "Close")

Call objDebugLogFile.Close

'start event source specific log
Set objDebugLogFile = objDebugLogObject.CreateTextFile(strLogFile, True)

Call LogInfo( "Start collect session")
Call LogInfo( "Parameters: OutFileName=" & strOutFileName + " LogDirectory=" & strLogDirectory & " BaseName=" + strBaseName & " PropsFilename=" & strPropsFilename & " SapVersion=" & strSapVersion)

If (objFso.FileExists(strPropsFilename)) Then
	Call LogInfo( "Found Property file: " & strPropsFilename )
	
	Set f = objFso.OpenTextFile(strPropsFilename, ForReading)
	data=f.ReadAll
	f.Close
	
	Call infoMessage(objDebugLogFile, "Content of Property file " & strPropsFilename  & data )

Else
	strRegKey = conRegKey+strLogDirectory
	Call LogInfo( "Property file not found, use data from registry '" & strRegKey &"'")
	
	' Get name of last logfile
	On Error Resume Next
	strLogLastFile = WshShell.RegRead(strRegKey)
	If Err.Number <> 0 Then
		strLogLastFile = strBaseName & "19700101"
	End If
	On Error GoTo 0	
	data = "last_file=" & strLogLastFile
End If

Call LogInfo(data )
Const SearchChar = "="
pos = InStr(data,SearchChar)
If(pos > 0 ) Then
  strLogLastFile = Mid(data,pos+1)
  pos = pos + Len(strBaseName)+1	
  strStartDate = Mid(data,pos)
 Else
  Call LogInfo( "Error reading properties, use default value 1970-Jan-01" )
  strLogLastFile = strBaseName & "19700101"
		
End If 

dtmEndDate =Now
strFinishDate =  year(dtmEndDate) _
		& Right("0" & month(dtmEndDate), 2) _
		& Right("0" & day(dtmEndDate), 2)	
strLastFileMerged = strLogLastFile

' Name of last opened logfile
Call LogInfo( "Last log file name: " & strLogLastFile )

' Date range
Call LogInfo( "Dates for processing: Start=" & strStartDate & " Finish=" & strFinishDate )

' Create Commandline for log merger
intCount = 0
Dim hastemp
hasTemp = False
' 2004jul29VL WE0406044 changed/sapmerge takes more than 4 arguments, first 3 are required
'                       and other arguments are optional. First required argument is SAP version,
'                       so pass it as first argument.
strCmdLine = conConcantCmd + " """ + strSapVersion + """ " + conTmpRawData
Set objFso = CreateObject("Scripting.FileSystemObject")
Set objFolder = objFso.GetFolder(strLogDirectory)
Call LogInfo( "Log Directory "+strLogDirectory+" opened" )
Set objFc  = objFolder.Files
Call LogInfo( "Looping through files" )
For Each objFile In objFc
  If objFile.name >= strLogLastFile Then
    Call LogInfo( "Processing File "+objFile.name )	
    If Len(strCmdline) + Len(objFile.Name) + Len(strLogDirectory) + 4 > 256 Then
      Call runLogMerger(strCmdline ) 'cmdline got to long  merge into .tmp file
      If hasTemp then
        Call objFso.DeleteFile(conTmpRawData & ".tmp")
      End If
      hasTemp = true
      Call objFso.moveFile(conTmpRawData,conTmpRawData & ".tmp")
      ' 2004jul29VL WE0406044 changed/pass SAP version as first argument
      strCmdLine = conConcantCmd + " """ + strSapVersion + """ " + _
		   conTmpRawData + " " + conTmpRawData + ".tmp"
      ' 2004jul29VL WE0406044 changed/end
    End if
    strCmdLine = strCmdLine + " " +""""+ strLogDirectory+"\"+objFile.Name+""""
    intCount = intCount + 1
    strLastFileMerged = objFile.name
  End If
Next
Call LogInfo( "Looping done." )

' Run logmerger
If intCount > 0 Then
  Call runLogMerger(strCmdline)
  If hasTemp then
    Call objFso.DeleteFile(conTmpRawData & ".tmp")
  End if
Else
  Call LogInfo("No new SAP log files found" )
End If

' Run logchopper
' IN2002jun10: sapchop.exe takes only 2 arguments
' 2004jul29VL WE0406044 changed/sapchop.exe takes 3 arguments, third argument is SAP version
' Andrew Tyapuhin 2005Dec13 QE050C007 changed/sapchop.exe takes 4 arguments, 4th argument is props file
strCmdLine = conChopperCmd _
           +" "+ strOutFileName _
           +" "+ conTmpRawData _
           +" """+ strSapVersion + """" _
           +" "+ strPropsFilename
           
' 2004jul29VL WE0406044 changed/end
'           +" "+ strOutFileName _
'           +" "+ conTmpRawData _
'           +" "+ strPropsFilename

Set f = objFso.GetFile(conTmpRawData)
Call LogInfo("Size of raw data file: " & f.Size & " bytes")


Call LogInfo("Starting sapchop.exe: " & strCmdLine)

intReturnCode = WshShell.Run(strCmdLine, 0, True)
Call LogInfo("sapchop return code : " & intReturnCode )
If intReturnCode <> 0 Then
    Call LogInfo("End session, New new properties will be not saved" )
    Call objDebugLogFile.Close
    Call wscript.quit(intReturnCode)
End If

' Delete temporary file, if present
'Set objFso = CreateObject("Scripting.FileSystemObject")
objFso.DeleteFile(conTmpRawData)



'save only date of last file in props file
Set f = objFso.OpenTextFile(strPropsFilename, ForAppending)
f.WriteLine  "last_file=" & strLastFileMerged
f.Close

' output new content on props file
Set f = objFso.OpenTextFile(strPropsFilename, ForReading)
data=f.ReadAll
f.Close
Call LogInfo("New content of props file " & strPropsFilename & " :"+ Chr(10)+ Chr(13) & data )	

'delete registry entry with old settings
Call LogInfo("Delete registry key '" & strRegKey & "'" )	
On Error Resume Next
WshShell.RegDelete strRegKey
On Error GoTo 0

' Write last file to registry
'WshShell.RegWrite strRegKey, strLastFileMerged

Call LogInfo("End of collect session, exit successfully" )

' Close Files and Quit script
Call objDebugLogFile.Close
Call wscript.quit(intReturnCode)


'============================
'
' infoMessage()
'
Sub infoMessage(outputFile, text)
  If conTracing Then
    wscript.echo Now & " " & text
  End If

  Call outputFile.WriteLine(Now & " " & text)

End Sub


'============================
'
' LogInfo()
'
Sub LogInfo(ByVal message)
    Call infoMessage(objDebugLogFile, message)
End Sub

'============================
'
' runLogMerger()
'
Sub runLogMerger(ByVal strCmdLine)
  Dim WshShell          ' The shell to direct commands to
  Set WshShell = wscript.CreateObject("WScript.Shell")
  Call LogInfo(strCmdLine)
  intReturnCode = WshShell.Run(strCmdLine, 0, True)
  Call infoMessage(objDebugLogFile, "cmd returns : " & intReturnCode )
  If intReturnCode <> 0 Then
    Call infoMessage(objDebugLogFile, "Error on " & conConcantCmd )
    Call objDebugLogFile.Close
    Call wscript.quit(intReturnCode)
  End If 
  Call infoMessage(objDebugLogFile, "Merging Successfull" )
End Sub
