/**********************************************************BeginFile************
* Copyright:  (c) 2001 by CONSUL risk management b.v.  All Rights Reserved
* Time-stamp: <010619 ide 09:59 sapmerge.c>
*
* Name:  sapmerge.c
* Purpose:
*   Merge 2 or more files to one larger file
*
* Notes:
*  Call as sapmerge.exe version outfile infile [infile...]
*
*  error :
*
* History:
*  010619 0.0.7 ide Created
*  VL 2004jul29 updated: WE0406044 map: proper SAP 4.7 support. Added support of
*                        4.7 version, added more effective error checking,
*                        fixed buffer overflow.
*  VL 2004sep10 updated: WE0406044 - ignore the last log record if that record
*                        is shorter than the expected length.
***********************************************************EndFile*************/
#include <sapmerge.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h> /* 2004jul29VL WE0406044 added/include for more informative
                    * error messages */


/* 2004jul29VL WE0406044 removed/start
int main(int argc, char *argv[])
   2004jul29VL WE0406044 removed/end */
static int merge(int argc, char *argv[], char *record, size_t length) /*
 * 2004jul29VL WE0406044 changed/main function has been changed */
{
  FILE  *logFile;
  FILE  *outFile;
  int    i  = 0;
  int    ret = 0; /* 2004jul29VL WE0406044 added/indicator of errors */

  /* 2004jul29VL WE0406044 commented/start */
  /* char   record[RECORD_LENGTH]; *//* RATS: ignore */
  /*
  if (argc < 3)
  {
    printf("Call as %s outfile infile [infile...]\n",argv[0]);
    return 1;
  }

  printf("program  : %s \n",argv[0]);
  printf("out file : %s \n",argv[1]);
  printf("logfile1 : %s \n",argv[2]);
  printf("logfile2 : %s \n",argv[3]);
 
  logFile  = NULL;
  outFile  = NULL;
  */
  /* 2004jul29VL WE0406044 commented/end */

  /* open output file */
  outFile = fopen(argv[2], "w");

  /* 2004jul29VL WE0406044 added/error checking added */
  if (outFile == NULL)
  {
    /* can't open output file */
    printf("ERROR: Can't open output file (%s): %s\n", argv[1], strerror(errno));
    return 1;
  }
  /* 2004jul29VL WE0406044 added/end */

  /* loop through extra arguments*/
  for ( i=3; i<argc; i++ )
  {
    /* open log file */
    printf("opening file %s\n",argv[i]);
    logFile = fopen(argv[i], "r");

    /* 2004jul29VL WE0406044 added/error checking added */
    if (NULL == logFile)
    {
      /* can't open input file */
      printf("ERROR: Can't open input file (%s): %s.\n", argv[i], strerror(errno));
      /* set error result and break this cycle to close output file and return error */
      ret = 1;
      break;
    }
    /* 2004jul29VL WE0406044 added/end */

    /* Pre read the first record */
    fgets(record, length, logFile); /* 2004jul29VL WE0406044 changed/pass length
                                     * argument received from the arguments
                                     * instead of constant value */
    /* loop through all remaining records */
    while  (0 == feof(logFile) )
    {
      /* .. write record, and set new value for latest timestamp */
      fprintf(outFile, "%s", record);

      /* read the next record */
      fgets(record, length, logFile); /* 2004jul29VL WE0406044 changed/pass length
                                       * argument received from the arguments
                                       * instead of constant value */

      if (strlen(record) < length - 1) break; /* 2004sep10VL WE0406044 added/ignore
                                               * the last log record if that record
                                               * is shorter than the expected length */
    }
    fclose(logFile); /* 2004jul29VL WE0406044 changed/close opened file */
  }

  /* close open files */
  /* if (NULL != logFile)  fclose(logFile); *//* 2004jul29VL WE0406044 removed */
  fclose(outFile); /* 2004jul29VL WE0406044 changed/close file without any checking. */

  /* return 0; *//* 2004jul29VL WE0406044 removed */
  return ret; /* 2004jul29VL WE0406044 changed/return the result of execution */
}

/*
 * 2004jul29VL WE0406044 added/main function to check arguments and parse
 *                       SAP version has been added.
 */
int main(int argc, char *argv[])
{
  if (argc < 4)
  {
    printf("Call as %s version outfile infile [infile...]\n",argv[0]);
    return 1;
  }

  printf("program  : %s \n",argv[0]);
  printf("sap vers : %s \n",argv[1]);
  printf("out file : %s \n",argv[2]);
  printf("logfile1 : %s \n",argv[3]);
  printf("logfile2 : %s \n",argv[4]);

  if (!strcmp(argv[1], "") || !strcmp(argv[1], "4.6"))
  {
    char record[SAP_46_REC_LENGTH+1]; /* RATS: ignore */
    return merge(argc, argv, record, SAP_46_REC_LENGTH);
  }
  else
  {
    char record[SAP_47_REC_LENGTH+1]; /* RATS: ignore */
    return merge(argc, argv, record, SAP_47_REC_LENGTH);
  }
}
/* 2004jul29VL WE0406044 added/end */

